SystemOrganization addCategory: #'Seaside-Tests-Flow-Unit'!
SystemOrganization addCategory: #'Seaside-Tests-Flow-Functional'!

WAFunctionalTest subclass: #WACallDuringRenderFunctionalTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

!WACallDuringRenderFunctionalTest methodsFor: 'rendering' stamp: 'jf 10/28/2008 10:15'!
renderContentOn: html
	html paragraph: [
		html anchor
			callback: [ self call: WACallDuringRenderTestComponent new ];
			with: 'This link'.
		html text: ' will call a Component which attempts to do a #call: in its #renderContentOn: method. This is not allowed and should trigger a descriptive error. With WAWalkbackErrorHandler enabled, the output should explain to the user the likely cause of the problem.' ]! !

WAFunctionalTest subclass: #WAFlowFunctionalTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

!WAFlowFunctionalTest methodsFor: 'actions' stamp: 'pmm 8/31/2008 14:32'!
goAnchors
	| component |
	1 to: 5 do: [ :each | 
		component := WAComponent new.
		component addMessage: [ :html | 
			html anchor
				callback: [ component answer ];
				with: each seasideString , ': ' , self stackDepth seasideString ] fixCallbackTemps.
		self call: component ]! !

!WAFlowFunctionalTest methodsFor: 'actions' stamp: 'pmm 8/31/2008 14:33'!
goButtons
	1 to: 5 do: [ :each |
		self inform: each seasideString, ': ', self stackDepth seasideString ]! !

!WAFlowFunctionalTest methodsFor: 'rendering' stamp: 'lr 5/22/2007 18:10'!
renderContentOn: html
	html paragraph: 'The following two anchors should trigger flows with 5 steps each. The stack should neither grow nor shrink. Backtracking and spawning of windows should properly work.'.
	html paragraph: [
		html anchor
			callback: [ self goAnchors ];
			with: 'go anchors'.
		html break.
		html anchor
			callback: [ self goButtons ];
			with: 'go buttons' ]! !

!WAFlowFunctionalTest methodsFor: 'private' stamp: 'jf 1/22/2009 02:43'!
stackDepth

	^WAPlatform current stackDepth! !

!WAPackage class methodsFor: '*seaside-tests-flow' stamp: 'lr 1/24/2009 21:27'!
seasideTestsFlow
	^ self new
		name: 'Seaside-Tests-Flow';
		description: 'Functional and unit tests for Seaside-Flow.';
		addDependency: 'Seaside-Flow';
		yourself! !

WAComponent subclass: #WACallDuringRenderTestComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

!WACallDuringRenderTestComponent methodsFor: 'rendering' stamp: 'jf 10/28/2008 10:13'!
renderContentOn: html
	self call: WAMiniCalendar new! !

WATask subclass: #WADecorationTask
	instanceVariableNames: 'component1 component2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

!WADecorationTask methodsFor: 'accessing' stamp: 'lr 11/28/2008 16:40'!
children
	"Even if we component1 and component2, we have to return them as children as otherwise their decorations are not properly backtracked. See WADecorationTest."
	
	^ Array with: component1 with: component2! !

!WADecorationTask methodsFor: 'running' stamp: 'lr 5/31/2008 10:36'!
go
	self call: component1.
	self call: component2! !

!WADecorationTask methodsFor: 'initialization' stamp: 'lr 5/31/2008 15:53'!
initialize
	super initialize.
	component1 := WAFormDialog new
		addMessage: 'Step 1';
		yourself.
	component2 := WAFormDialog new
		addMessage: 'Step 2';
		yourself! !

WATask subclass: #WAFunctionalTaskTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

WAFunctionalTaskTest subclass: #WAConvenienceFunctionalTest
	instanceVariableNames: 'cheese'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

!WAConvenienceFunctionalTest methodsFor: 'controlling' stamp: 'lr 10/28/2007 14:42'!
chooseCheese
	cheese := self
		chooseFrom: #('Greyerzer' 'Tilsiter' 'Sbrinz')
		caption: 'What''s your favorite Cheese?'.
	cheese isNil ifTrue: [ self chooseCheese ]! !

!WAConvenienceFunctionalTest methodsFor: 'controlling' stamp: 'lr 10/28/2007 14:42'!
confirmCheese
	^self confirm: 'Is ', cheese,  ' your favorite cheese?'! !

!WAConvenienceFunctionalTest methodsFor: 'controlling' stamp: 'pmm 6/30/2007 12:24'!
go
	[ self chooseCheese.
	  self confirmCheese ] whileFalse.
	self informCheese! !

!WAConvenienceFunctionalTest methodsFor: 'controlling' stamp: 'lr 10/28/2007 14:42'!
informCheese
	self inform: 'Your favorite cheese is ', cheese, '.'! !

!WAConvenienceFunctionalTest methodsFor: 'accessing' stamp: 'pmm 8/29/2008 16:05'!
states
	^Array with: self! !

WAFunctionalTaskTest subclass: #WAExceptionFunctionalTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

!WAExceptionFunctionalTest methodsFor: 'processing' stamp: 'pmm 11/1/2007 21:32'!
go
	[ (self confirm: 'Raise an exception?')
		ifTrue: [ self error: 'foo' ] ]
			on: Error
			do: [ :error | self inform: 'Caught: ', error description ]! !

WATask subclass: #WANestedTransaction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

!WANestedTransaction commentStamp: 'pmm 9/2/2007 00:08' prior: 0!
A WANestedTransaction is a test that uses two nested #isolate: blocks!

!WANestedTransaction methodsFor: 'processing' stamp: 'lr 10/28/2007 14:42'!
go
	self inform: 'Before parent txn'.
	self isolate:
			[self inform: 'Inside parent txn'.
			self isolate: [self inform: 'Inside child txn'].
			self inform: 'Outside child txn'].
	self inform: 'Outside parent txn'! !

TestCase subclass: #WAContinuationTest
	instanceVariableNames: 'tmp tmp2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Unit'!

!WAContinuationTest methodsFor: 'utilities' stamp: 'lr 3/24/2008 11:47'!
callcc: aBlock
	^ WAContinuation currentDo: aBlock! !

!WAContinuationTest methodsFor: 'tests' stamp: 'lr 1/5/2007 17:26'!
testBlockEscape
	| x |
	tmp := 0.
	x := [ tmp := tmp + 1. tmp2 value ].
	self callcc: [ :cc | tmp2 := cc. x value ].
	tmp2 := [ ].
	x value.
	self assert: tmp = 2! !

!WAContinuationTest methodsFor: 'tests' stamp: 'dkh 7/5/2007 12:27'!
testBlockTemps
	| y |
	#(1 2 3) do: [ :i |
		| x |
		x := i.
		tmp ifNil: [ tmp2 := (self callcc: [ :cc | tmp := cc. [ :q | ] ]) ].
		tmp2 value: x.
		x := 17 ].
	y := (self callcc: [ :cc | tmp value: cc. 42 ]).
	self assert: y = 1! !

!WAContinuationTest methodsFor: 'tests' stamp: 'lr 10/28/2007 14:42'!
testBlockVars
	| continuation |
	tmp := 0.
	tmp := (self callcc: [ :cc | continuation := cc. 0 ]) + tmp.
	tmp2
		ifNotNil: [ tmp2 value ]
		ifNil: [
			#(1 2 3) do: [ :i |
				self callcc: [ :cc | tmp2 := cc. continuation value: i ] ] ].
	self assert: tmp = 6! !

!WAContinuationTest methodsFor: 'tests' stamp: 'lr 10/28/2007 14:42'!
testComprehension
	"What should this print out?

	| yin yang |
	yin := [ :x | Transcript cr. x ] value: Continuation current.
	yang := [ :x | Transcript nextPut: $*. x ] value: Continuation current.
	yin value: yang"! !

!WAContinuationTest methodsFor: 'tests' stamp: 'lr 1/5/2007 17:27'!
testMethodTemps
	| i continuation |
	i := 0.
	i := i + (self callcc: [:cc | continuation := cc. 1]).
	self assert: i ~= 3.
	i = 2 ifFalse: [ continuation value: 2 ]! !

!WAContinuationTest methodsFor: 'tests' stamp: 'lr 12/3/2008 19:09'!
testNestedWhileTrue
	| account continuation accountValues  |
	true ifTrue: [^self].
	tmp := 0.
	account := nil.
	accountValues := Array new: 2.
	tmp2 := Array new: 2.
    [:finalVal | 
      [:initialVal | 
        ["This puts account two levels up"
          account := self callcc: [:oc | continuation := oc. initialVal ].
		  tmp2 at: tmp + 1 put: account.
		  account isNil] whileTrue.
	    tmp := tmp + 1.
	    accountValues at: tmp put: account.
	    continuation notNil
		  ifTrue: [
			continuation value: finalVal]] value: 1"initialVal"] 
      value: 2"finalVal".
	self assert: tmp = 2.
	self assert: account = 2.

! !

!WAContinuationTest methodsFor: 'tests' stamp: 'lr 1/5/2007 18:22'!
testReentrant
	| assoc |
	assoc := self callcc: [ :cc | cc -> 0 ].
	assoc value: assoc value + 1.
	self assert: assoc value ~= 5.
	assoc value = 4
		ifFalse: [ assoc key value: assoc ]! !

!WAContinuationTest methodsFor: 'tests' stamp: 'lr 1/5/2007 17:28'!
testSimpleCallCC
	| x continuation |
	x := self callcc: [ :cc | continuation := cc. false ].
	x ifFalse: [ continuation value: true ].
	self assert: x! !

!WAContinuationTest methodsFor: 'tests' stamp: 'pmm 1/10/2009 23:00'!
testSimpleCallCCWithPossibleArgument
	| x continuation |
	x := self callcc: [ :cc | continuation := cc. false ].
	x ifFalse: [ continuation valueWithPossibleArguments: (Array with:  true) ].
	self assert: x! !

!WAContinuationTest methodsFor: 'tests' stamp: 'lr 1/5/2007 17:28'!
testSimplestCallCC
	| x |
	x := self callcc: [ :cc | cc value: true ].
	self assert: x! !

!WAContinuationTest methodsFor: 'tests' stamp: 'lr 12/3/2008 19:10'!
testWhileTrue
	| account continuation accountValues  |
	true ifTrue: [^self].
	tmp := 0.
	account := nil.
	accountValues := Array new: 2.
	tmp2 := Array new: 2.
	[account := (self callcc: [:oc | continuation := oc. 1 ]).
		tmp2 at: tmp + 1 put: account.
		account isNil] whileTrue.
	tmp := tmp + 1.
	accountValues at: tmp put: account.
	continuation notNil
		ifTrue: [
			continuation value: 2].
	self assert: tmp = 2.
	self assert: account = 2.! !

TestCase subclass: #WAPartialContinuationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Unit'!

!WAPartialContinuationTest methodsFor: 'private' stamp: 'lr 1/16/2009 23:23'!
callcc: aBlock
	^ WAPartialContinuation marker: self marker do: aBlock! !

!WAPartialContinuationTest methodsFor: 'private' stamp: 'lr 1/24/2009 21:26'!
mark: aBlock
	^ aBlock value! !

!WAPartialContinuationTest methodsFor: 'private' stamp: 'lr 1/15/2009 22:54'!
marker
	^ self class compiledMethodAt: #mark:! !

!WAPartialContinuationTest methodsFor: 'testing' stamp: 'lr 1/16/2009 23:25'!
testMarkerOnCall
	| seen |
	seen := false.
	self 
		should: [ self callcc: [ :cc | seen := true ] ]
		raise: Error.
	self deny: seen.
	
	seen := false.
	self
		shouldnt: [ self mark: [ self callcc: [ :cc | seen := true ] ] ]
		raise: Error.
	self assert: seen.
	
	seen := false.
	! !

!WAPartialContinuationTest methodsFor: 'testing' stamp: 'lr 1/16/2009 23:27'!
testMarkerOnValue
	| kk seen |
	self mark: [ 
		self callcc: [ :cc | kk := cc ]. 
		seen := true ].
	
	seen := false.
	self 
		should: [ kk value ]
		raise: Error.
	self deny: seen.
	
	seen := false.
	self
		shouldnt: [ self mark: [ kk value ] ]
		raise: Error.
	self assert: seen! !

!WAPartialContinuationTest methodsFor: 'testing' stamp: 'lr 1/16/2009 23:35'!
testNestedEvaluation
	| k1 k2 |
	self assert: (self mark: [ 
		2 * (self callcc: [ :c1 | 
			k1 := c1. 
			3 * (self callcc: [ :c2 |
				k2 := c2.
				4 ]) ]) ])
					= 24.
					
	self assert: (self mark: [
		k1 value: 5 ])
			= 10.
	self assert: (self mark: [
		k2 value: 5 ])
			= 30! !

!WAPartialContinuationTest methodsFor: 'testing' stamp: 'lr 1/16/2009 23:36'!
testReentrant
	| kk |
	self assert: (self mark: [ 
		2 * (self callcc: [ :cc | 
			kk := cc. 2 ]) ]) 
				= 4.
	
	self assert: (self mark: [
		kk value: 3 ]) 
			= 6.
	self assert: (self mark: [
		kk value: 4 ]) 
			= 8.
	self assert: (self mark: [
		kk value: 5 ]) 
			= 10! !

!WAPartialContinuationTest methodsFor: 'testing' stamp: 'lr 1/16/2009 23:44'!
testSimple
	self assert: (self mark: [ self callcc: [ :cc | cc value: true ] ])! !
